;; -*- no-byte-compile: t; lexical-binding: nil -*-
(define-package "ccc" "20260322.1316"
  "Buffer local cursor color control library."
  ()
  :url "https://github.com/skk-dev/ddskk"
  :commit "29a290bb62fa58bd8aad330c54049b7a7ac8449a"
  :revdesc "29a290bb62fa"
  :keywords '("cursor")
  :authors '(("Masatake YAMATO" . "masata-y@is.aist-nara.ac.jp")))
